package v2

import (
	"encoding/json"
	"time"

	"github.com/aquasecurity/trivy/pkg/types"
	"golang.org/x/xerrors"

	"github.com/future-architect/vuls/constant"
	"github.com/future-architect/vuls/contrib/trivy/pkg"
	"github.com/future-architect/vuls/models"
)

// ParserV2 is a parser for scheme v2
type ParserV2 struct {
}

// Parse trivy's JSON and convert to the Vuls struct
func (p ParserV2) Parse(vulnJSON []byte) (result *models.ScanResult, err error) {
	var report types.Report
	if err = json.Unmarshal(vulnJSON, &report); err != nil {
		return nil, err
	}

	scanResult, err := pkg.Convert(report.Results)
	if err != nil {
		return nil, err
	}

	if err := setScanResultMeta(scanResult, &report); err != nil {
		return nil, err
	}
	return scanResult, nil
}

func setScanResultMeta(scanResult *models.ScanResult, report *types.Report) error {
	const trivyTarget = "trivy-target"
	for _, r := range report.Results {
		if pkg.IsTrivySupportedOS(r.Type) {
			scanResult.Family = r.Type
			scanResult.ServerName = r.Target
			scanResult.Optional = map[string]interface{}{
				trivyTarget: r.Target,
			}
		} else if pkg.IsTrivySupportedLib(r.Type) {
			if scanResult.Family == "" {
				scanResult.Family = constant.ServerTypePseudo
			}
			if scanResult.ServerName == "" {
				scanResult.ServerName = "library scan by trivy"
			}
			if _, ok := scanResult.Optional[trivyTarget]; !ok {
				scanResult.Optional = map[string]interface{}{
					trivyTarget: r.Target,
				}
			}
		}
		scanResult.ScannedAt = time.Now()
		scanResult.ScannedBy = "trivy"
		scanResult.ScannedVia = "trivy"
	}

	if _, ok := scanResult.Optional[trivyTarget]; !ok {
		return xerrors.Errorf("scanned images or libraries are not supported by Trivy. see https://aquasecurity.github.io/trivy/dev/vulnerability/detection/os/, https://aquasecurity.github.io/trivy/dev/vulnerability/detection/language/")
	}
	return nil
}
